/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <string.h>
#include <components.h>

xml_dtc_hash *xml_dtc_handler_components_loop_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  /* TODO: factorize */
  xmlChar *attr, *type = NULL;
  /*  xml_dtc_hash *result;*/

  xml_dtc_hash *good_handlers;

  good_handlers = (xml_dtc_hash *) xml_dtc_hash_get(handlers, (char *) node->name);

  /*  result = xml_dtc_container_new(sitemap, (char *) node->name);*/

  attr = xmlGetProp(node, "default");

  node = node->children;

  while (node) {
    /* Ignore Blank Nodes */
    if (! xmlIsBlankNode(node)) {
      /* Test if node is an element or something else (PI, Comment...) */
      if (node->type != XML_ELEMENT_NODE) {
	/* Comments are allowed */
	if (node->type == XML_COMMENT_NODE) {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found comment");
	} else {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Element or comment expected");
	  return NULL;
	}
      } else {
	/* Check if namespace is good */
	if ((node->ns != node->parent->ns) && (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE) != 0)) {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "expecting %s namespace, but got %s", XML_DTC_SITEMAP_NAMESPACE,
		      (node->ns != NULL) ? (char *) node->ns->href : "none");
	  return NULL;
	}
	/* Now we know that namespace is ok, we can check and initialize all stuff */
	/* Search a handler for this node */
	/* TODO: avoid memleak with attr when an error occurs */
	{
	  xml_dtc_handler *tmp;
	  xml_dtc_object *object;
	  xmlChar *attr;
	  attr = xmlGetProp(node, "src");
	  if ((tmp = (xml_dtc_handler *)xml_dtc_hash_get(good_handlers, (char *)attr))) {
	    if (!type) type = (xmlChar *)node->name;
	    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found handler for '%s' %s", attr, node->name);
	    if ((object = tmp->handle(tmp, handlers, sitemap, node)) == NULL)
	      return NULL;
	    /*	  else
		  xml_dtc_container_add(result, object);*/
	  } else {
	    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "No handler for '%s' %s", attr, node->name);
	    return NULL;
	  }
	  free(attr);
	}
      }
    }
    node = node->next;
  }

  /*  xml_dtc_container_set_preferred_by_key(result, attr);*/
  xml_dtc_sitemap_set_preferred_by_key(sitemap, type, attr);
  free(attr);

  /*  return result;*/
  return (xml_dtc_hash *)handler;
}

int _xml_dtc_caller_free(xml_dtc_caller *caller) {
  if (caller) {
    if (caller->resolved == 0)
      free(caller->component.name);
    else
      xml_dtc_object_unlink((xml_dtc_object *)caller->component.object);
    if (caller->params)
      xml_dtc_hash_free(caller->params);
  }
  return 0;
}
