/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef __HASH_H__
#define __HASH_H__

#include <objects.h>
#include <string.h>

typedef void *(*xml_dtc_hash_link_element_callback)(void *element);
typedef void (*xml_dtc_hash_unlink_element_callback)(void *element);
typedef char *(*xml_dtc_hash_link_key_callback)(char *key);
typedef void (*xml_dtc_hash_unlink_key_callback)(char *key);

typedef struct _xml_dtc_hash_struct xml_dtc_hash_struct;
struct _xml_dtc_hash_struct {
  char *key;
  void *element;
};

typedef struct _xml_dtc_hash xml_dtc_hash;
struct _xml_dtc_hash {
  xml_dtc_resource *resource;
  int num;
  int size;
  xml_dtc_hash_link_key_callback link_key;
  xml_dtc_hash_unlink_key_callback unlink_key;
  xml_dtc_hash_link_element_callback link_element;
  xml_dtc_hash_unlink_element_callback unlink_element;
  xml_dtc_hash_struct *list;
  //  xml_dtc_hash_struct **hash;
  xml_dtc_hash_struct *dfault;
};

int xml_dtc_hash_free(xml_dtc_hash *hash);

xml_dtc_hash *xml_dtc_hash_new(const xml_dtc_resource *resource, xml_dtc_hash_unlink_key_callback unlink_key, xml_dtc_hash_link_key_callback link_key, xml_dtc_hash_unlink_element_callback unlink_element, xml_dtc_hash_link_element_callback link_element);

#define xml_dtc_hash_new_cast(resource, unlink_key, link_key, unlink_element, link_element) xml_dtc_hash_new(resource, (xml_dtc_hash_unlink_key_callback) (unlink_key), (xml_dtc_hash_link_key_callback) (link_key), (xml_dtc_hash_unlink_element_callback) (unlink_element), (xml_dtc_hash_link_element_callback) (link_element))

#define xml_dtc_object_hash_new(resource) xml_dtc_hash_new_cast(resource, free, strdup, xml_dtc_object_unlink, xml_dtc_object_link)

#define xml_dtc_string_hash_new(resource) xml_dtc_hash_new_cast(resource, free, strdup, free, strdup)

#define xml_dtc_value_hash_new(resource) xml_dtc_hash_new_cast(resource, free, strdup, NULL, NULL)

void *xml_dtc_hash_get(xml_dtc_hash *hash, const char *key);

int xml_dtc_hash_add(xml_dtc_hash *hash, char *key, void *element);

#define xml_dtc_hash_add_cast(hash, key, element) xml_dtc_hash_add(hash, key, (void *) (element))

#define xml_dtc_object_hash_add(hash, element) xml_dtc_hash_add(hash, ((xml_dtc_object *)(element))->name, (void *) (element))

#define xml_dtc_string_hash_add(hash, key, element) xml_dtc_hash_add(hash, (char *) key, (void *) (element))

int xml_dtc_hash_set_default(xml_dtc_hash *hash, const char *key);

#endif /* __HASH_H__ */
