/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <string.h>
#include "xml-dtc.h"

#ifdef OBJECT_DEBUG
#define SIZE_MOD 64
#define xml_dtc_all_objects ((xml_dtc_object **)xml_dtc._private)
#define xml_dtc_all_objects_len xml_dtc.links
#endif

xml_dtc_object *xml_dtc_object_new(const xml_dtc_resource *resource, const char *name, const char *type) {
  xml_dtc_object *result;
  xml_dtc_type *t;

  if (!(t = xml_dtc_type_get(type))) {
    xml_dtc_msg(resource, XML_DTC_MSG_ERROR, "Attempt to create non existent object type '%s'.", type);
    return NULL;
  }

  result = (xml_dtc_object *) malloc(t->size);
  memset(result, 0, t->size);
  
  result->resource = xml_dtc_resource_link(resource);
  
  result->type = t;
  
  if (name)
    result->name = strdup(name);
  
#ifdef OBJECT_DEBUG
  if ((xml_dtc_all_objects_len % SIZE_MOD) == 0) {
    xml_dtc_all_objects = realloc(xml_dtc_all_objects, sizeof(xml_dtc_object *) * (xml_dtc_all_objects_len + SIZE_MOD));
  }
  xml_dtc_all_objects[xml_dtc_all_objects_len++] = result;
#endif

  return result;
}

xml_dtc_object *xml_dtc_object_link(xml_dtc_object *object) {
  if (object)
    object->links++;

  return object;
}

int xml_dtc_object_unlink(xml_dtc_object *object) {
  if (object) {
    // xml_dtc_msg(object->resource, XML_DTC_MSG_ERROR, "Unlinking '%s' %s.", object->name, object->type->name);
    if (object->links-- > 1) {
      //xml_dtc_msg(object->resource, XML_DTC_MSG_ERROR, "'%s' %s still has %d links.", object->name, object->type->name, object->links);
      return 1;
    } else {
      xml_dtc_type *t;

      t = object->type;

      //xml_dtc_msg(object->resource, XML_DTC_MSG_ERROR, "Destroying '%s' %s.", object->name, object->type->name);
      while (t) {
	if (t->free)
	  t->free(object);
	t = t->parent;
      }


#ifndef OBJECT_DEBUG
      free(object->name);
#endif

      if (object->resource)
	xml_dtc_resource_unlink(object->resource);
    
#ifdef OBJECT_DEBUG
      object->links = -42;
#else
      free(object);
#endif
      return 0;
    }
  }
  return -1;
}

#ifdef OBJECT_DEBUG
void xml_dtc_objects_dump() {
  int i;
  for (i = 0; i < xml_dtc_all_objects_len; i++) {
    if (xml_dtc_all_objects[i]->links != -42)
      printf("%s %s, with %d links\n", xml_dtc_all_objects[i]->name, xml_dtc_all_objects[i]->type->name, xml_dtc_all_objects[i]->links);
  }
}
#endif
