/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef __REQUEST_H__
#define __REQUEST_H__

typedef struct _xml_dtc_request xml_dtc_request;

#include <objects.h>
#include <xml-dtc.h>
#include <sitemap.h>
#include <mappings.h>
#include <hash.h>
#include <libxml/tree.h>

struct _xml_dtc_request {
  xml_dtc_resource *resource;
  int internal;
  int status;
  char *method, *uri, *path_info;
  char *query_string;
  xml_dtc_hash *headers, *parameters, *attributes, *cookies, *newcookies, *out_headers;
  struct {
    void *headers;
    void *data;
  } data;
  xmlDocPtr xmltree;
  xmlChar *result;
  char *mime_type;
  char *encoding;
  int content_length;
  xml_dtc_mappings *mappings;
  struct {
    char *type;
    char *message;
    char *description;
    const xml_dtc_object *sender;
  } error;
  xml_dtc_sitemap *sitemap;

  int (*send_http_headers)(xml_dtc_request *);
  int (*register_headers)(xml_dtc_request *);
  int (*register_parameters)(xml_dtc_request *);
  int (*register_cookies)(xml_dtc_request *);
  int (*write)(void *, char *, int);
  void *context;
};

int xml_dtc_request_free(xml_dtc_request *request);

xml_dtc_request *xml_dtc_request_new(const xml_dtc_resource *resource, const char *method, const char *uri, const char *path_info);

int xml_dtc_request_set_error(xml_dtc_request *request, const xml_dtc_object *sender, const char *type, const char *message, const char *description);

int xml_dtc_request_serialize_error(xml_dtc_request *request);

xml_dtc_hash *xml_dtc_request_get_parameters(xml_dtc_request *request);

char *xml_dtc_request_get_parameter(xml_dtc_request *request, const char *name);

int xml_dtc_request_set_parameter(xml_dtc_request *request, const char *name, const char *value);

int xml_dtc_request_set_parameters_from_query_string(xml_dtc_request *request, const char *query_string);

xml_dtc_hash *xml_dtc_request_get_attributes(xml_dtc_request *request);

char *xml_dtc_request_get_attribute(xml_dtc_request *request, const char *name);

int xml_dtc_request_set_attribute(xml_dtc_request *request, const char *name, const char *value);

xml_dtc_hash *xml_dtc_request_get_headers(xml_dtc_request *request);

char *xml_dtc_request_get_header(xml_dtc_request *request, const char *name);

int xml_dtc_request_set_header(xml_dtc_request *request, const char *name, const char *value);

xml_dtc_hash *xml_dtc_request_get_cookies(xml_dtc_request *request);

char *xml_dtc_request_get_cookie(xml_dtc_request *request, const char *name);

int xml_dtc_request_set_cookie(xml_dtc_request *request, const char *name, const char *value);

int xml_dtc_request_set_out_header(xml_dtc_request *request, const char *name, const char *value);

int xml_dtc_request_apply(xml_dtc_request *request, xml_dtc_list *list);

#endif /* __REQUEST_H__ */
