/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "xml-dtc.h"

void xml_dtc_resource_default_callback (void *context, void *channel, const char *format, va_list va) {
  vfprintf(channel, format, va);
  fprintf(channel, "\n");
}

xml_dtc_resource *xml_dtc_resource_new() {
  xml_dtc_resource *result;

  result = (xml_dtc_resource *) malloc(sizeof(xml_dtc_resource));
  memset(result, 0, sizeof(xml_dtc_resource));

#ifdef DEBUG
  xml_dtc_resource_set_msg_verbosity(result, XML_DTC_MSG_DEBUG);
#else
  xml_dtc_resource_set_msg_verbosity(result, XML_DTC_MSG_ERROR);
#endif
  xml_dtc_resource_set_msg_callback(result, xml_dtc_resource_default_callback);
  xml_dtc_resource_set_all_msg_channels(result, (void *) stderr);
  return result;
}

xml_dtc_resource *xml_dtc_resource_link(const xml_dtc_resource *resource) {
  if (resource)
    ((xml_dtc_resource *)resource)->links++;

  return (xml_dtc_resource *)resource;
}

int xml_dtc_resource_unlink(xml_dtc_resource *resource) {
  if (resource) {
    if (resource->links-- > 1) {
      return 1;
    } else {
      free(resource);
      return 0;
    }
  }
  return -1;
}

void xml_dtc_msg(const xml_dtc_resource *resource, xml_dtc_msg_level level, const char *format, ...) {
  va_list va;
  va_start(va, format);
  if (resource) {
    if (level <= resource->msg_verbosity) {
      void *channel;
      channel = resource->msg_channels[level];
      ((xml_dtc_msg_callback)resource->msg_callback)(resource->msg_context, channel, format, va);
    }
  } else {
    if (level <= XML_DTC_MSG_WARNING) {
      vfprintf(stderr, format, va);
    }
  }
}
