/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef __XML_DTC_H__
#define __XML_DTC_H__

extern struct _xml_dtc xml_dtc;

extern struct _xml_dtc_types xml_dtc_types;

typedef struct _xml_dtc_handler xml_dtc_handler;

typedef struct _xml_dtc_runner xml_dtc_runner;

#include <resource.h>
#include <hash.h>
#include <list.h>
#include <sitemap.h>
#include <request.h>
#include <components.h>
#include <libxml/tree.h>

typedef xml_dtc_object * (*xml_dtc_handler_callback) (xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node);

typedef int (*xml_dtc_runner_callback) (xml_dtc_request *request);

struct _xml_dtc_handler {
  void *_private;
  char *name;
  char *parent_node;
  xml_dtc_handler_callback handle;
};

struct _xml_dtc_runner {
  void *_private;
  char *name;
  char *parent_node;
  xml_dtc_runner_callback run;
};

struct _xml_dtc {
  xml_dtc_resource *resource;
  xml_dtc_hash *handlers;
  xml_dtc_list *initializers;
  xml_dtc_list *runners;
  xml_dtc_list *hooks;
  xml_dtc_hash *modules;
  xml_dtc_hash *contexts;
};

struct _xml_dtc_types {
  xml_dtc_resource *resource;
  int num;
  int size;
  xml_dtc_type *hash;
};

int xml_dtc_init_modules();

int xml_dtc_init(const xml_dtc_resource *resource);

int xml_dtc_cleanup();

int xml_dtc_register_initializer(const char *name, xml_dtc_handler_callback callback, void *private_data);

int xml_dtc_register_handler(const char *name, const char *parent, xml_dtc_handler_callback callback, void *private_data);

int xml_dtc_register_runner(const char *name, xml_dtc_runner_callback callback, void *private_data);

typedef int (*xml_dtc_hook) (xml_dtc_caller *caller, xml_dtc_request *request);

int xml_dtc_register_hook(xml_dtc_hook callback);

xml_dtc_type *xml_dtc_type_get(const char *name);

int xml_dtc_register_type(const char *name, int size, xml_dtc_object_free_callback free, const char *parent);

#endif /* __XML_DTC_H__ */
