/*
 * This module provides redirection to the XML-DTC framework
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This module is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this module; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <modules.h>
#include <xml-dtc.h>
#include <mappings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

typedef struct _xml_dtc_redirect_caller xml_dtc_redirect_caller;

typedef struct _xml_dtc_redirector xml_dtc_redirector;

struct _xml_dtc_redirector {
  XML_DTC_COMPONENT_STANDARD_DEF;
};

struct _xml_dtc_redirect_caller {
  XML_DTC_CALLER_STANDARD_DEF;
  xmlChar *uri;
};

static int _xml_dtc_redirect_caller_free(xml_dtc_redirect_caller *caller) {
  if (caller->uri)
    xmlFree(caller->uri);
  return 0;
}

static int xml_dtc_redirect(xml_dtc_redirect_caller *caller, xml_dtc_request *request) {
  char *uri;

  uri = xml_dtc_resolve_mappings(request->mappings, caller->uri);

  request->status = 302;
  request->result = strdup("");
  xml_dtc_request_set_out_header(request, "Location", uri);
  free(uri);

  return 0;
}

static xml_dtc_object *xml_dtc_handler_redirector_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xml_dtc_redirector *result;

  result = (xml_dtc_redirector *)xml_dtc_object_new(sitemap->resource, "redirector", "redirector");
  result->handle = xml_dtc_redirect;

  xml_dtc_sitemap_add(sitemap, (xml_dtc_object *)result);
  xml_dtc_sitemap_set_preferred_by_key(sitemap, "redirector", "redirector");
  
  return (xml_dtc_object *)handler;
}

static xml_dtc_object *xml_dtc_handler_redirect_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xml_dtc_redirect_caller *result;
  result = (xml_dtc_redirect_caller *)xml_dtc_object_new(sitemap->resource, "redirector", "redirect-caller");
  result->uri = xmlGetProp(node, "uri");

  return (xml_dtc_object *)result;
}

static int xml_dtc_redirect_module_register(void) {
  xml_dtc_register_initializer("redirector", (xml_dtc_handler_callback) xml_dtc_handler_redirector_callback, NULL);
  xml_dtc_register_handler("redirect-to", "pipeline", (xml_dtc_handler_callback) xml_dtc_handler_redirect_callback, NULL);
  xml_dtc_register_type("redirect-caller", sizeof(xml_dtc_redirect_caller), (xml_dtc_object_free_callback) _xml_dtc_redirect_caller_free, "caller");
  xml_dtc_register_type("redirector", sizeof(xml_dtc_redirector), NULL, "component");
  return 0;
}

xml_dtc_module xml_dtc_redirect_module = {
  XML_DTC_MODULE_STANDARD_STUFF,
  xml_dtc_redirect_module_register
};
