/*
 * This module provides resources to the XML-DTC framework
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This module is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this module; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <modules.h>
#include <xml-dtc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

typedef struct _xml_dtc_resource_component_caller xml_dtc_resource_component_caller;

typedef struct _xml_dtc_resource_component xml_dtc_resource_component;
struct _xml_dtc_resource_component {
  XML_DTC_COMPONENT_STANDARD_DEF;
  xml_dtc_list *sub_pipeline;
};

struct _xml_dtc_resource_component_caller {
  XML_DTC_CALLER_STANDARD_DEF;
};

static int _xml_dtc_resource_component_free(xml_dtc_resource_component *resource) {
  if (resource->sub_pipeline)
    xml_dtc_list_free(resource->sub_pipeline);
  return 0;
}

/*const char *XML_DTC_SITEMAP_NAMESPACE = "http://apache.org/cocoon/sitemap/1.0";
  const char *XML_DTC_SITEMAP_ROOT_NODE = "sitemap";*/

static xml_dtc_hash *xml_dtc_handler_loop_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  /*  xml_dtc_container *result;*/
  xml_dtc_hash *good_handlers;

  /* TODO: fix the memleak when returning NULL */

  good_handlers = (xml_dtc_hash *) xml_dtc_hash_get(handlers, (char *) node->name);

  /*  result = xml_dtc_container_new(sitemap, (char *) node->name);*/

  /* TODO: check the root node */
  /*
  if (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE)
      || xmlStrcmp(node->name, (const xmlChar *) XML_DTC_SITEMAP_ROOT_NODE)) {
    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "File doesn't have the right root node.");
    return NULL;
    }*/

  node = node->children;

  while (node) {
    /* Ignore Blank Nodes */
    if (! xmlIsBlankNode(node)) {
      /* Test if node is an element or something else (PI, Comment...) */
      if (node->type != XML_ELEMENT_NODE) {
	/* Comments are allowed */
	if (node->type == XML_COMMENT_NODE) {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found comment");
	} else {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Element or comment expected");
	  return NULL;
	}
      } else {
	/* Check if namespace is good */
	if ((node->ns != node->parent->ns) && (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE) != 0)) {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "expecting %s namespace, but got %s", XML_DTC_SITEMAP_NAMESPACE,
		      (node->ns != NULL) ? (char *) node->ns->href : "none");
	  return NULL;
	}
	/* Now we know that namespace is ok, we can check and initialize all stuff */
	/* Search a handler for this node */
	{
	  xml_dtc_handler *tmp;
	  xml_dtc_object *object;
	  if ((tmp = (xml_dtc_handler *)xml_dtc_hash_get(good_handlers, (char *)node->name))) {
	    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found handler for %s", node->name);
	    if ((object = tmp->handle(tmp, handlers, sitemap, node)) == NULL)
	      return NULL;
	    /*	  else
		  xml_dtc_container_add(result, object);*/
	  } else {
	    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "No handler for %s", node->name);
	    return NULL;
	  }
	}
      }
    }
    node = node->next;
  }

  /*  return result;*/
  return (xml_dtc_hash *)handler;
}

static xml_dtc_list *xml_dtc_handler_pipeline_loop_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xml_dtc_list *result;
  xml_dtc_hash *good_handlers;

  /* TODO: fix the memleak when returning NULL */

  good_handlers = (xml_dtc_hash *) xml_dtc_hash_get(handlers, (char *) node->name);

  result = xml_dtc_object_list_new(sitemap->resource);

  node = node->children;

  while (node) {
    /* Ignore Blank Nodes */
    if (! xmlIsBlankNode(node)) {
      /* Test if node is an element or something else (PI, Comment...) */
      if (node->type != XML_ELEMENT_NODE) {
        /* Comments are allowed */
        if (node->type == XML_COMMENT_NODE) {
          xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found comment");
        } else {
          xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Element or comment expected");
          return NULL;
        }
      } else {
        /* Check if namespace is good */
        if ((node->ns != node->parent->ns) && (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE) != 0)) {
          xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "expecting %s namespace, but got %s", XML_DTC_SITEMAP_NAMESPACE,
                      (node->ns != NULL) ? (char *) node->ns->href : "none");
          return NULL;
        }
        /* Now we know that namespace is ok, we can check and initialize all stuff */
        /* Search a handler for this node */
        {
          xml_dtc_handler *tmp;
          xml_dtc_object *object;
          if ((tmp = (xml_dtc_handler *)xml_dtc_hash_get(good_handlers, (char *)node->name))) {
            xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found handler for %s", node->name);
            if ((object = tmp->handle(tmp, handlers, sitemap, node)) == NULL)
              return NULL;
            else
              xml_dtc_list_push_cast(result, object);
          } else {
            xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "No handler for %s", node->name);
            return NULL;
          }
        }
      }
    }
    node = node->next;
  }

  return result;
}

static xml_dtc_object *xml_dtc_handler_resource_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xmlChar *attr;
  xml_dtc_resource_component *result;

  attr = xmlGetProp(node, "name");
  result = (xml_dtc_resource_component *)xml_dtc_object_new(sitemap->resource, attr, "Resource");
  result->handle = handler->_private;
  free(attr);

  xmlNodeSetName(node, "pipeline");
  result->sub_pipeline = xml_dtc_handler_pipeline_loop_callback(handler, handlers, sitemap, node);

  xml_dtc_sitemap_add(sitemap, (xml_dtc_object *) result);
  return (xml_dtc_object *)handler;
}

static xml_dtc_object *xml_dtc_handler_call_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xml_dtc_resource_component_caller *result;
  xmlChar *attr;

  attr = xmlGetProp(node, "resource");
  result = (xml_dtc_resource_component_caller *)xml_dtc_object_new(sitemap->resource, "Resource", "resource-caller");
  result->component.name = attr;

  node = node->children;

  while (node) {
    /* Ignore Blank Nodes */
    if (! xmlIsBlankNode(node)) {
      /* Test if node is an element or something else (PI, Comment...) */
      if (node->type != XML_ELEMENT_NODE) {
        /* Comments are allowed */
        if (node->type == XML_COMMENT_NODE) {
          xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found comment");
        } else {
          xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Element or comment expected");
          return NULL;
        }
      } else {
        /* Check if namespace is good */
        if ((node->ns != node->parent->ns) && (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE) != 0)) {
          xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "expecting %s namespace, but got %s", XML_DTC_SITEMAP_NAMESPACE,
                      (node->ns != NULL) ? (char *) node->ns->href : "none");
          return NULL;
        }
        /* Now we know that namespace is ok, we can check and initialize all stuff */
        /* Search a handler for this node */
        {
	  if (xmlStrcmp(node->name, "parameter") == 0) {
	    xmlChar *tmp1, *tmp2;
	    if (!result->params) {
	      result->params = xml_dtc_string_hash_new(result->resource);
	    }
	    xml_dtc_string_hash_add(result->params, tmp1 = xmlGetProp(node, "name"), tmp2 = xmlGetProp(node, "value"));
	    xmlFree(tmp1);
	    xmlFree(tmp2);
	  }
        }
      }
    }
    node = node->next;
  }

  return (xml_dtc_object *)result;
}

static int xml_dtc_call(xml_dtc_resource_component_caller *caller, xml_dtc_request *request) {
  xml_dtc_mappings *mappings;
  xml_dtc_hash *hash;
  int ret;

  mappings = request->mappings;
  request->mappings = xml_dtc_mappings_new(request->resource);
  
  hash = xml_dtc_mappings_new_hash(request->mappings);
  if (caller->params)
    for (ret = 0; ret < caller->params->num; ret++) {
      if (caller->params->list[ret].key) {
	char *tmp = xml_dtc_resolve_mappings(mappings, caller->params->list[ret].element);
	xml_dtc_string_hash_add(hash, caller->params->list[ret].key, tmp);
	free(tmp);
      }
    }
  ret = xml_dtc_request_apply(request, ((xml_dtc_resource_component *)caller->component.object)->sub_pipeline);
  xml_dtc_mappings_free(request->mappings);
  request->mappings = mappings;
  return ret;
}

static int xml_dtc_resources_module_register(void) {
  xml_dtc_register_handler("resources", "sitemap", (xml_dtc_handler_callback) xml_dtc_handler_loop_callback, NULL);
  xml_dtc_register_handler("resource", "resources", (xml_dtc_handler_callback) xml_dtc_handler_resource_callback, xml_dtc_call);
  xml_dtc_register_handler("call", "pipeline", (xml_dtc_handler_callback) xml_dtc_handler_call_callback, NULL);
  xml_dtc_register_type("Resource", sizeof(xml_dtc_resource_component), (xml_dtc_object_free_callback) _xml_dtc_resource_component_free, "component");
  xml_dtc_register_type("resource-caller", sizeof(xml_dtc_resource_component_caller), NULL, "caller");
  return 0;
}

xml_dtc_module xml_dtc_resources_module = {
  XML_DTC_MODULE_STANDARD_STUFF,
  xml_dtc_resources_module_register
};
