/*
 * This module provides sitemap to the XML-DTC framework
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This module is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this module; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <modules.h>
#include <xml-dtc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

/*const char *XML_DTC_SITEMAP_NAMESPACE = "http://apache.org/cocoon/sitemap/1.0";
  const char *XML_DTC_SITEMAP_ROOT_NODE = "sitemap";*/

static xml_dtc_hash *xml_dtc_handler_loop_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  /*  xml_dtc_container *result;*/
  xml_dtc_hash *good_handlers;

  /* TODO: fix the memleak when returning NULL */

  good_handlers = (xml_dtc_hash *) xml_dtc_hash_get(handlers, (char *) node->name);

  /*  result = xml_dtc_container_new(sitemap, (char *) node->name);*/

  /* TODO: check the root node */
  /*
  if (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE)
      || xmlStrcmp(node->name, (const xmlChar *) XML_DTC_SITEMAP_ROOT_NODE)) {
    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "File doesn't have the right root node.");
    return NULL;
    }*/

  node = node->children;

  while (node) {
    /* Ignore Blank Nodes */
    if (! xmlIsBlankNode(node)) {
      /* Test if node is an element or something else (PI, Comment...) */
      if (node->type != XML_ELEMENT_NODE) {
	/* Comments are allowed */
	if (node->type == XML_COMMENT_NODE) {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found comment");
	} else {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Element or comment expected");
	  return NULL;
	}
      } else {
	/* Check if namespace is good */
	if ((node->ns != node->parent->ns) && (xmlStrcmp(node->ns->href, XML_DTC_SITEMAP_NAMESPACE) != 0)) {
	  xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "expecting %s namespace, but got %s", XML_DTC_SITEMAP_NAMESPACE,
		      (node->ns != NULL) ? (char *) node->ns->href : "none");
	  return NULL;
	}
	/* Now we know that namespace is ok, we can check and initialize all stuff */
	/* Search a handler for this node */
	{
	  xml_dtc_handler *tmp;
	  xml_dtc_object *object;
	  if ((tmp = (xml_dtc_handler *)xml_dtc_hash_get(good_handlers, (char *)node->name))) {
	    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_DEBUG, "Found handler for %s", node->name);
	    if ((object = tmp->handle(tmp, handlers, sitemap, node)) == NULL)
	      return NULL;
	    /*	  else
		  xml_dtc_container_add(result, object);*/
	  } else {
	    xml_dtc_msg(sitemap->resource, XML_DTC_MSG_ERROR, "No handler for %s", node->name);
	    return NULL;
	  }
	}
      }
    }
    node = node->next;
  }

  /*  return result;*/
  return (xml_dtc_hash *)handler;
}

static int xml_dtc_sitemap_module_register(void) {
  xml_dtc_register_handler("sitemap", "xml-dtc", (xml_dtc_handler_callback) xml_dtc_handler_loop_callback, NULL);
  xml_dtc_register_handler("components", "sitemap", (xml_dtc_handler_callback) xml_dtc_handler_loop_callback, NULL);

  return 0;
}

xml_dtc_module xml_dtc_sitemap_module = {
  XML_DTC_MODULE_STANDARD_STUFF,
  xml_dtc_sitemap_module_register
};
